/*
 * mainframe.cpp
 *
 * Main frame in mqlqtwx.
 * mqlqtwx is a graphical front-end for querying Emdros databases.
 *
 * Ulrik Petersen
 * Created: 4/13-2005
 * Last update: 4/11-2009
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2005-2009  Ulrik Sandborg-Petersen
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation, license version 2.  
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *   02111-1307 USA
 *
 *
 *   Special exception
 *   =================
 * 
 *   In addition, as a special exception, Ulrik Petersen, the
 *   copyright holder of Emdros, gives permission to link Emdros, in
 *   whole or in part, with the libraries which are normally
 *   distributed with:
 *   
 *   a) Sun's Java platform,
 *   b) Python, 
 *   c) Jython,
 *   d) Ruby, and/or 
 *   e) Perl 
 *   f) PostgreSQL
 *   g) OpenSSL
 *
 *   (or with modified versions of these), and to distribute linked
 *   combinations including both Emdros, in whole or in part, and one
 *   or more of the libraries normally distributed with (a)-(g) above.
 *
 *   Please note: This gives you special rights concerning the
 *   libraries which normally accompany the above pieces of software.
 *   It gives you no special rights concerning software that you write
 *   yourself.  You must obey the GNU General Public License in all
 *   respects for all of the code used other than the libraries
 *   normally distributed with (a)-(g) above.
 *
 *   If you modify this file, you may extend this exception to your
 *   version of the file, but you are not obligated to do so. If you
 *   do not wish to do so, delete this exception statement from your
 *   version.
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 *
 *   Website
 *   =======
 *
 *   Emdros has a website here:
 *
 *   http://emdros.org
 *
 *
 *
 **************************************************************************/

#if defined(__GNUG__) && !defined(__APPLE__)
#pragma implementation "mainframe.h"
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include <wx/wx.h>
#endif

////@begin includes
// #include <wx/stdpaths.h> // Not available in 2.4.2
#include <wx/utils.h> // Instead of stdpaths
#include <wx/filename.h>
#include <wx/filedlg.h>
#include <wx/strconv.h>
#include <wx/fs_zip.h>
#include <wx/clipbrd.h>
#include <wx/dataobj.h>
#include <fstream>
#include <sstream>
#include "exec.h"
#include <wxutil_emdros.h>
#include <prefix_emdros.h>
#include "qtconnwizard.h"

////@end includes

#include "mainframe.h"
#include "conndlg.h"
#include "mqlqtwx.h"
#include <monads.h>

////@begin XPM images
#include <connect.xpm>
#include <new.xpm>
#include <open.xpm>
#include <save.xpm>
#include <copy.xpm>
#include <cut.xpm>
#include <paste.xpm>
#include <flash.xpm>
#include <stop.xpm>
#include <clear.xpm>
////@end XPM images


#define min(X,Y)  ((X < Y) ? X : Y)



/*!
 * MainFrame type definition
 */

IMPLEMENT_CLASS( MainFrame, wxFrame )
	IMPLEMENT_CLASS( MyTextCtrl, wxTextCtrl )

/*!
 * MainFrame event table definition
 */

	BEGIN_EVENT_TABLE( MainFrame, wxFrame )
////@begin MainFrame event table entries
	EVT_MENU(wxID_ABOUT, MainFrame::OnAbout)
	EVT_MENU(wxID_HELP, MainFrame::OnHelpContents)
	EVT_MENU(ID_MENU_HELP_MQL_CHEAT_SHEET, MainFrame::OnHelpMQLCheatSheet)
	EVT_MENU(wxID_NEW, MainFrame::OnFileNew)
	EVT_MENU(wxID_OPEN, MainFrame::OnFileOpen)
	EVT_MENU(wxID_SAVE, MainFrame::OnFileSave)
	EVT_MENU(wxID_SAVEAS, MainFrame::OnFileSaveAs)
	EVT_MENU(ID_MENU_FILE_SAVE_OUTPUT_AS, MainFrame::OnFileSaveOutputAs)
	EVT_MENU(wxID_CUT, MainFrame::OnEditCut)
	EVT_MENU(wxID_COPY, MainFrame::OnEditCopy)
	EVT_MENU(wxID_PASTE, MainFrame::OnEditPaste)
	EVT_MENU(ID_MENU_TOOLS_NEW_CONNECTION, MainFrame::OnToolsNewConnection)
	EVT_MENU(ID_MENU_TOOLS_EXECUTE_QUERY, MainFrame::OnToolsExecuteQuery)
	EVT_MENU(ID_MENU_TOOLS_STOP_QUERY, MainFrame::OnToolsStopQuery)
	EVT_MENU(ID_MENU_TOOLS_CONFIGURE, MainFrame::OnToolsConfigure)
	EVT_MENU(wxID_EXIT, MainFrame::OnFileExit)
	EVT_MENU(ID_MENU_FILE_CLEAR_OUTPUT_WINDOW, MainFrame::OnClearOutputWindow)

	EVT_UPDATE_UI(wxID_ABOUT, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(wxID_HELP, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(ID_MENU_HELP_MQL_CHEAT_SHEET, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(wxID_NEW, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(wxID_OPEN, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(wxID_SAVE, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(wxID_SAVEAS, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(ID_MENU_FILE_SAVE_OUTPUT_AS, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(wxID_CUT, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(wxID_COPY, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(wxID_PASTE, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(ID_MENU_TOOLS_NEW_CONNECTION, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(ID_MENU_TOOLS_EXECUTE_QUERY, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(ID_MENU_TOOLS_STOP_QUERY, MainFrame::OnUpdateUIExec)
	EVT_UPDATE_UI(ID_MENU_TOOLS_CONFIGURE, MainFrame::OnUpdateUINoExec)
	EVT_UPDATE_UI(wxID_EXIT, MainFrame::OnUpdateUINoExec)

	EVT_MAXIMIZE(MainFrame::OnMaximize)
	EVT_SIZE(MainFrame::OnSize)

////@end MainFrame event table entries

	END_EVENT_TABLE()




	BEGIN_EVENT_TABLE( MyTextCtrl, wxTextCtrl )
	EVT_CHAR(MyTextCtrl::OnChar)
	END_EVENT_TABLE()


wxString getInputAreaFontName(Configuration *pConf)
{
	wxString inputAreaFontName;
	std::string str("input_area_font_name");
	if (pConf != 0 && pConf->has_key(str)) {
		std::string input_area_font_name = pConf->getValues(str)[0];
		inputAreaFontName = wxString(input_area_font_name.c_str(), wxConvUTF8);
	} else {
#ifdef __WXMSW__
#if wxUSE_UNICODE
		inputAreaFontName = wxString(wxT("Arial MS Unicode"));
#else
		inputAreaFontName = wxString(wxT("Courier New"));
#endif
#else  // Not Windows
		inputAreaFontName = wxString(wxT("Courier"));
#endif
	}
	return inputAreaFontName;
}




/*!
 * MainFrame constructors
 */

MainFrame::MainFrame( )
		: m_pEE(0), m_pEW(0), m_pConf(0), m_pOut(0), m_bExecuting(false)
{
	readConfig();
}

MainFrame::~MainFrame()
{
	writeConfig();
	delete m_pEE;
	delete m_pEW;
	delete m_pConf;
	delete m_pOut;
}

MainFrame::MainFrame( wxWindow* parent, 
		      wxWindowID id, 
		      const wxString& caption, 
		      const wxPoint& pos, 
		      const wxSize& size, 
		      long style )
	: m_pEE(0), m_pEW(0), m_pConf(0), m_pOut(0), m_bExecuting(false)
{
	readConfig();

	Create( parent, id, caption, pos, size, style );

	// Set default style of text controls
	if (m_pOut != 0) {
		wxTextAttr inputAreaTextAttr(*wxBLACK, wxNullColour,
					     wxFont(m_pOut->getInputAreaFontSize(), 
						    wxMODERN, 
						    wxNORMAL, 
						    wxNORMAL, 
						    FALSE, 
						    m_pOut->getInputAreaFontName()),
					     wxTEXT_ALIGNMENT_LEFT);
		inputAreaTextAttr.SetFlags(wxTEXT_ATTR_TEXT_COLOUR
					   | wxTEXT_ATTR_FONT_FACE
					   | wxTEXT_ATTR_FONT_SIZE
					   | wxTEXT_ATTR_FONT_WEIGHT
					   | wxTEXT_ATTR_FONT_ITALIC
					   | wxTEXT_ATTR_FONT_UNDERLINE
					   | wxTEXT_ATTR_ALIGNMENT);
		m_pEditWindow->SetDefaultStyle(inputAreaTextAttr);
		// m_pEditWindow->SetStyle(0, m_pEditWindow->GetLastPosition(), inputAreaTextAttr);
		m_pEditWindow->Clear();
	}
	m_pOut->print(std::string((const char*) (wxString(wxT("Welcome.  Press the first button ")
							  wxT("(\"connect to database\") in ")
							  wxT("order to specify a ")
							  wxT("configuration file (*.cfg).\n")
							  wxT("See the User's Guide for ")
							  wxT("details on how to write a ")
							  wxT("configuration file. ")
							  wxT("Press F1 to get the User's Guide.\n")).mb_str(wxConvUTF8))));

}

/*!
 * MainFrame creator
 */

bool MainFrame::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
	////@begin MainFrame member initialisation
	m_ctrlSplitterQueryResults = NULL;
	m_ctrlSplitterLeftRight = NULL;
	m_pEditWindow = NULL;
	m_pResultsWindow = NULL;
	m_ctrlToolBar = NULL;
	////@end MainFrame member initialisation

	////@begin MainFrame creation
	wxFrame::Create( parent, id, caption, pos, size, style );

	CreateControls();
	Centre();

	////@end MainFrame creation
	return TRUE;
}

void MainFrame::SetCurFileName(const wxString& strFileName)
{
	m_strCurFileName = strFileName;
	if (m_strCurFileName.IsEmpty()) {
		SetTitle(wxT("Emdros Query Tool"));
	} else {
		SetTitle(wxT("EQT: ") + m_strCurFileName);
	}
}

/*!
 * Control creation for MainFrame
 */

void MainFrame::CreateControls()
{    
	////@begin MainFrame content construction

	SetCurFileName(wxT(""));
	MainFrame* itemFrame1 = this;

	m_ctrlToolBar = new wxToolBar( itemFrame1, ID_TOOLBAR, wxDefaultPosition, wxDefaultSize, wxTB_FLAT|wxTB_HORIZONTAL );
	m_ctrlToolBar->SetToolBitmapSize(wxSize(48,48));

	wxBitmap connectBitmap(itemFrame1->GetBitmapResource(wxT("connect.xpm")));
	m_ctrlToolBar->AddTool(ID_MENU_TOOLS_NEW_CONNECTION, _T(""), connectBitmap, _T("Connect to database"), wxITEM_NORMAL);
	m_ctrlToolBar->AddSeparator();

	wxBitmap itemtool21Bitmap(itemFrame1->GetBitmapResource(wxT("new.xpm")));
	m_ctrlToolBar->AddTool(wxID_NEW, _T(""), itemtool21Bitmap, _T("New query"), wxITEM_NORMAL);
	wxBitmap itemtool22Bitmap(itemFrame1->GetBitmapResource(wxT("open.xpm")));
	m_ctrlToolBar->AddTool(wxID_OPEN, _T(""), itemtool22Bitmap, _T("Open query"), wxITEM_NORMAL);
	wxBitmap itemtool23Bitmap(itemFrame1->GetBitmapResource(wxT("save.xpm")));
	m_ctrlToolBar->AddTool(wxID_SAVE, _T(""), itemtool23Bitmap, _T("Save query"), wxITEM_NORMAL);
	m_ctrlToolBar->AddSeparator();
	wxBitmap itemtool25Bitmap(itemFrame1->GetBitmapResource(wxT("copy.xpm")));
	m_ctrlToolBar->AddTool(wxID_COPY, _T(""), itemtool25Bitmap, _T("Copy to clipboard"), wxITEM_NORMAL);
	wxBitmap itemtool26Bitmap(itemFrame1->GetBitmapResource(wxT("cut.xpm")));
	m_ctrlToolBar->AddTool(wxID_CUT, _T(""), itemtool26Bitmap, _T("Cut to clipboard"), wxITEM_NORMAL);
	wxBitmap itemtool27Bitmap(itemFrame1->GetBitmapResource(wxT("paste.xpm")));
	m_ctrlToolBar->AddTool(wxID_PASTE, _T(""), itemtool27Bitmap, _T("Paste from clipboard"), wxITEM_NORMAL);
	m_ctrlToolBar->AddSeparator();
	wxBitmap itemtool28Bitmap(itemFrame1->GetBitmapResource(wxT("flash.xpm")));
	m_ctrlToolBar->AddTool(ID_MENU_TOOLS_EXECUTE_QUERY, _T("Execute query"), itemtool28Bitmap, _T(""), wxITEM_NORMAL);
	wxBitmap itemtool29Bitmap(itemFrame1->GetBitmapResource(wxT("stop.xpm")));
	m_ctrlToolBar->AddTool(ID_MENU_TOOLS_STOP_QUERY, _T("Stop execution"), itemtool29Bitmap, _T(""), wxITEM_NORMAL);
	wxBitmap itemtool30Bitmap(itemFrame1->GetBitmapResource(wxT("clear.xpm")));
	m_ctrlToolBar->AddTool(ID_MENU_FILE_CLEAR_OUTPUT_WINDOW, _T("Clear output window"), itemtool30Bitmap, _T(""), wxITEM_NORMAL);

	m_ctrlToolBar->Realize();
	itemFrame1->SetToolBar(m_ctrlToolBar);

	wxStatusBar* itemStatusBar27 = new wxStatusBar( itemFrame1, ID_STATUSBAR, wxST_SIZEGRIP|wxNO_BORDER );
	itemStatusBar27->SetFieldsCount(2);
	itemFrame1->SetStatusBar(itemStatusBar27);

	m_ctrlSplitterLeftRight = new wxSplitterWindow( itemFrame1, ID_SPLITTERWINDOW_VERTICAL, wxDefaultPosition, wxSize(100, 100), wxSP_3DBORDER|wxSP_3DSASH|wxNO_BORDER );

	m_pSchemaTree = new wxTreeCtrl(m_ctrlSplitterLeftRight, ID_TREECTRL_SCHEMA, wxDefaultPosition, wxDefaultSize, wxTR_DEFAULT_STYLE, wxDefaultValidator, wxT("Schema tree"));
	/*
	if (ShowToolTips())
		m_pSchemaTree->SetToolTip(wxT("The schema is shown here if you are connected to a database."));
	*/

	m_ctrlSplitterQueryResults = new wxSplitterWindow( m_ctrlSplitterLeftRight, ID_SPLITTERWINDOW, wxDefaultPosition, wxSize(100, 100), wxSP_3DBORDER|wxSP_3DSASH|wxNO_BORDER );

	m_pEditWindow = new MyTextCtrl( m_ctrlSplitterQueryResults, 
					ID_TEXTCTRL_EDIT_WINDOW,
					_T(""), 
					wxDefaultPosition, 
					wxDefaultSize, 
					wxTE_MULTILINE | wxTE_RICH2 | wxTE_PROCESS_TAB );
	m_pEditWindow->SetHelpText(wxT("For writing your query or configuration."));
	/*
	if (ShowToolTips())
		m_pEditWindow->SetToolTip(wxT("Write your query or configuration here."));
	*/

#ifdef __WXMSW__
#if wxUSE_UNICODE
	m_pEditWindow->SetFont(wxFont(11, wxMODERN, wxNORMAL, wxNORMAL, FALSE, wxT("Courier New")));
#else
	m_pEditWindow->SetFont(wxFont(11, wxMODERN, wxNORMAL, wxNORMAL, FALSE, wxT("Courier New")));
#endif

#else  // Not windows

#if defined(wxUSE_UNICODE)
	m_pEditWindow->SetFont(wxFont(13, wxMODERN, wxNORMAL, wxNORMAL, FALSE, wxT("Courier New"), wxFONTENCODING_ISO8859_1)); // FIXME: We should do it properly!
#else
	m_pEditWindow->SetFont(wxFont(13, wxMODERN, wxNORMAL, wxNORMAL, FALSE, wxT("Courier New"), wxFONTENCODING_ISO8859_1)); // FIXME: We should do it properly!
#endif
#endif


	m_pResultsWindow = new WXILLayoutCanvas( m_ctrlSplitterQueryResults, wxDefaultPosition, wxDefaultSize, 0, m_pConf);
	m_pResultsWindow->SetHelpText(wxT("The results will be shown here."));
	/*
	if (ShowToolTips())
		m_pResultsWindow->SetToolTip(wxT("The results will be shown here."));
	*/

	m_ctrlSplitterQueryResults->SplitHorizontally(m_pEditWindow, m_pResultsWindow, 25);

	m_ctrlSplitterLeftRight->SplitVertically(m_ctrlSplitterQueryResults, m_pSchemaTree, 200);

	wxMenuBar* menuBar = new wxMenuBar;
	wxMenu* itemMenu3 = new wxMenu;
	itemMenu3->Append(wxID_NEW, wxT("New..."), _T(""), wxITEM_NORMAL);
	itemMenu3->Append(wxID_OPEN, wxT("&Open"), wxT("Opens a query or a configuration file."), wxITEM_NORMAL);
	itemMenu3->Append(wxID_SAVE, wxT("&Save"), wxT("Saves the current query."), wxITEM_NORMAL);
	itemMenu3->Append(wxID_SAVEAS, wxT("Save &as"), wxT("Saves the current query as..."), wxITEM_NORMAL);
	itemMenu3->Append(ID_MENU_FILE_SAVE_OUTPUT_AS, wxT("Sa&ve output as HTML"), wxT("Saves the current output as HTML."), wxITEM_NORMAL);
	itemMenu3->Append(ID_MENU_FILE_CLEAR_OUTPUT_WINDOW, wxT("&Clear output window"), wxT("Clear output window"), wxITEM_NORMAL);
	itemMenu3->AppendSeparator();
	itemMenu3->Append(wxID_EXIT, wxT("E&xit"), _T(""), wxITEM_NORMAL);
	menuBar->Append(itemMenu3, wxT("&File"));
	wxMenu* itemMenu10 = new wxMenu;
	itemMenu10->Append(wxID_COPY, wxT("Copy\tCtrl-C"), wxT("Copy the current selection to clipboard."), wxITEM_NORMAL);
	itemMenu10->Append(wxID_CUT, wxT("Cut\tCtrl-X"), wxT("Cuts the current selection to clipboard."), wxITEM_NORMAL);
	itemMenu10->Append(wxID_PASTE, wxT("Paste\tCtrl-V"), wxT("Pastes the current clipboard contents."), wxITEM_NORMAL);
	menuBar->Append(itemMenu10, wxT("&Edit"));
	wxMenu* itemMenu14 = new wxMenu;
	itemMenu14->Append(ID_MENU_TOOLS_NEW_CONNECTION, wxT("New &Connection..."), _T(""), wxITEM_NORMAL);
	itemMenu14->Append(ID_MENU_TOOLS_EXECUTE_QUERY, wxT("E&xecute Query"), wxT("Executes the current query."), wxITEM_NORMAL);
	/*
	itemMenu14->Append(ID_MENU_TOOLS_CONFIGURE, wxT("Configure..."), wxT("Lets you create or edit a configuration file."), wxITEM_NORMAL);
	*/
	menuBar->Append(itemMenu14, wxT("&Tools"));
	wxMenu* itemMenu18 = new wxMenu;
	itemMenu18->Append(wxID_HELP, wxT("&Help Contents\tF1"), _T("Brings up the help viewer"), wxITEM_NORMAL);
	itemMenu18->Append(ID_MENU_HELP_MQL_CHEAT_SHEET, wxT("MQL query language &Cheat sheet\tF2"), _T("Brings up the help viewer with the MQL query language cheat sheet"), wxITEM_NORMAL);
#ifndef __WXMAC__
	// On Mac OS X, the About menu-item is moved to the Application Menu,
	// so we don't want a separator here.
	itemMenu18->AppendSeparator();
#endif
	itemMenu18->Append(wxID_ABOUT, wxT("A&bout Emdros Query Tool..."), _T("Brings up the About box"), wxITEM_NORMAL);
	menuBar->Append(itemMenu18, wxT("&Help"));
	itemFrame1->SetMenuBar(menuBar);

	////@end MainFrame content construction
	m_pOut = new WXILQTOutput(m_pResultsWindow);

	wxCommandEvent evt;
	OnFileNew(evt);
}

/*!
 * Should we show tooltips?
 */

bool MainFrame::ShowToolTips()
{
	return TRUE;
}

/*!
 * Get bitmap resources
 */

wxBitmap MainFrame::GetBitmapResource( const wxString& name )
{
	// Bitmap retrieval
	////@begin MainFrame bitmap retrieval
	if (name == wxT("new.xpm")) {
		wxBitmap bitmap(new_xpm);
		return bitmap;
	} else if (name == wxT("open.xpm")) {
		wxBitmap bitmap(open_xpm);
		return bitmap;
	} else if (name == wxT("save.xpm")) {
		wxBitmap bitmap(save_xpm);
		return bitmap;
	} else if (name == wxT("copy.xpm")) {
		wxBitmap bitmap(copy_xpm);
		return bitmap;
	} else if (name == wxT("cut.xpm")) {
		wxBitmap bitmap(cut_xpm);
		return bitmap;
	} else if (name == wxT("paste.xpm")) {
		wxBitmap bitmap(paste_xpm);
		return bitmap;
	} else if (name == wxT("stop.xpm")) {
		wxBitmap bitmap(stop_xpm);
		return bitmap;
	} else if (name == wxT("flash.xpm")) {
		wxBitmap bitmap(flash_xpm);
		return bitmap;
	} else if (name == wxT("clear.xpm")) {
		wxBitmap bitmap(clear_xpm);
		return bitmap;
	} else if (name == wxT("connect.xpm")) {
		wxBitmap bitmap(connect_xpm);
		return bitmap;
	}
	return wxNullBitmap;
	////@end MainFrame bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon MainFrame::GetIconResource( const wxString& name )
{
	// Icon retrieval
	////@begin MainFrame icon retrieval
	return wxNullIcon;
	////@end MainFrame icon retrieval
}

void MainFrame::OnMaximize(wxMaximizeEvent& event)
{
	ResizeSplitters();

	// Pass it higher up in the hierarchy
	event.Skip();
}

void MainFrame::OnSize(wxSizeEvent& event)
{
	ResizeSplitters();

	// Pass it higher up in the hierarchy
	event.Skip();
}

void MainFrame::ResizeSplitters(void)
{
	wxSize clientSize = GetClientSize();

	int height = clientSize.GetHeight();
	int width = clientSize.GetWidth();

	int schematreewidth = min(300, int(width*0.4));
	int leftright_sashposition = width - schematreewidth;

	int halfheight = height / 2;
	int queryresults_sashposition = height - halfheight;

	m_ctrlSplitterQueryResults->SetSashPosition(queryresults_sashposition, TRUE);
	m_ctrlSplitterLeftRight->SetSashPosition(leftright_sashposition, TRUE);

	// Make sure we refresh this frame.
	Refresh();
}


void MainFrame::OnAbout(wxCommandEvent& WXUNUSED(event) )
{
	(void)wxMessageBox(_T("Emdros Query Tool version 1.3.0 from Emdros version")
			   wxT(EMDROS_VERSION)
			   _T("\n\nCopyright (C) 2005-2009 Ulrik Sandborg-Petersen\n\nMade available under the GNU General Public License,\nversion 2.\n\nEmdros Query Tool comes with NO WARRANTY WHATSOEVER.\n\nSee  http://emdros.org/qrytool  for more information.\n\nPlease visit  http://emergence.dk  for commercial support and licensing."), 
			   _T("About Emdros Query Tool"));
}

void MainFrame::OnHelpContents(wxCommandEvent& WXUNUSED(event) )
{
	wxHtmlHelpController *help;
	help = new wxHtmlHelpController(wxHF_DEFAULT_STYLE);

	help->AddBook(wxFileName(EmdrosQueryToolApp::wxHelpPrefix() + wxT("EQTUsersGuide.htb")));
   
	help->DisplayContents();
}

void MainFrame::OnHelpMQLCheatSheet(wxCommandEvent& WXUNUSED(event) )
{
	wxHtmlHelpController *help;
	help = new wxHtmlHelpController(wxHF_DEFAULT_STYLE);

	help->AddBook(wxFileName(EmdrosQueryToolApp::wxHelpPrefix() + wxT("EQTUsersGuide.htb")));
   
	help->DisplaySection(3100);
}

void MainFrame::OnFileNew(wxCommandEvent& event)
{
	ClearTextCtrls();
	SetCurFileName(wxT(""));
	this->Refresh();
}

void MainFrame::OnClearOutputWindow(wxCommandEvent& event)
{
	ClearOutputWindow();
}

void MainFrame::OnFileOpen(wxCommandEvent& event)
{
	//wxStandardPaths stdpaths;
	wxFileDialog dlgFileDialog(this,
				   wxT("Choose query file"),
				   //stdpaths.GetUserDataDir(),
				   m_strQueryFileDir,
				   wxT(""),
				   wxT("MQL Queries (*.mql)|*.mql|TXT files (*.txt)|*.txt|All files (*.*)|*.*"),
				   wxFD_OPEN | wxFD_FILE_MUST_EXIST);
	int nReturn;
	if ((nReturn = dlgFileDialog.ShowModal()) == wxID_OK) {
		SetCurFileName(dlgFileDialog.GetPath());

		// Set m_strQueryFileDir to the path-part of the 
		// gotten filename.
		wxFileName fn(m_strCurFileName);
		m_strQueryFileDir = fn.GetPath();
		
		ReadQuery();
	}
}

void MainFrame::OnFileSave(wxCommandEvent& event)
{
	if (m_strCurFileName.IsEmpty()) {
		OnFileSaveAs(event);
	} else {
		SaveQuery();
	}
}

void MainFrame::OnFileSaveAs(wxCommandEvent& event)
{
	if (m_strCurFileName.empty()) {
		SetCurFileName(wxT("query.mql"));
	}
	wxString strQueryFileDir;
	wxString strQueryFileFile;
	wxFileName fn(m_strCurFileName);
	if (fn.IsAbsolute()) {
		strQueryFileDir = fn.GetPath();
		strQueryFileFile = fn.GetFullName();
	} else {
		strQueryFileDir = ::wxGetHomeDir();
		strQueryFileFile = wxT("query.mql");
	}

	m_strQueryFileDir = fn.GetPath();
	//wxStandardPaths stdpaths;
	wxFileDialog dlgFileDialog(this,
				   wxT("Choose filename to save as"),
				   strQueryFileDir,
				   strQueryFileFile,
				   wxT("MQL Queries (*.mql)|*.mql|CFG files (*.cfg)|*.cfg|TXT files (*.txt)|*.txt|All files (*)|*"),
				   wxFD_SAVE);
	int nReturn;
	if ((nReturn = dlgFileDialog.ShowModal()) == wxID_OK) {
		SetCurFileName(dlgFileDialog.GetPath());

		// Set m_strQueryFileDir to the path-part of the 
		// gotten filename.
		wxFileName fn(m_strCurFileName);
		m_strQueryFileDir = fn.GetPath();

		SaveQuery();
	}
}

void MainFrame::OnFileSaveOutputAs(wxCommandEvent& event)
{
	wxString strOutputFileName = wxT("output.html");

	wxFileDialog dlgFileDialog(this,
				   wxT("Choose filename to save output as HTML"),
				   m_strQueryFileDir,
				   strOutputFileName,
				   wxT("HTML files (*.html)|*.html|All files (*)|*"),
				   wxFD_SAVE);
	int nReturn;
	if ((nReturn = dlgFileDialog.ShowModal()) == wxID_OK) {
		strOutputFileName = dlgFileDialog.GetPath();

		// Set m_strQueryFileDir to the path-part of the 
		// gotten filename.
		wxFileName fn(m_strCurFileName);
		m_strQueryFileDir = fn.GetPath();

		SaveOutputAreaAsHTML(strOutputFileName);
	}
}

void MainFrame::OnEditCopy(wxCommandEvent& event)
{
	wxTextCtrl *pFocusTextCtrl = GetFocusTextCtrl();
	if (pFocusTextCtrl == NULL) {
		// So it may be the tree control
		TreeControlCopyToClipboard(m_pSchemaTree);
	} else {
		pFocusTextCtrl->Copy();
	}
}


void MainFrame::TreeControlCopyToClipboard(wxTreeCtrl* pTree)
{
	// Note: If I ever add the wxTR_MULTIPLE style, I will have to use
	// GetSelections() instead!
	wxTreeItemId treeItemId = pTree->GetSelection();

	// Check that it is OK
	if (treeItemId.IsOk()) {
		wxString label = pTree->GetItemText(treeItemId);

		// If there is a space in there, it means it is
		// a 'OT : feature_type' string, and we only want
		// the object type.  So extract everything up to (but not
		// including) the first space, or the whole string
		// if there is no space.
		wxString toCopy = label.BeforeFirst(wxT(' '));

		// Copy text to clipboard as text.
		if (wxTheClipboard->Open()) {
			wxTheClipboard->SetData(new wxTextDataObject(toCopy));
			wxTheClipboard->Close();
		}
	}
}

void MainFrame::OnEditCut(wxCommandEvent& event)
{
	wxTextCtrl *pFocusTextCtrl = GetFocusTextCtrl();
	if (pFocusTextCtrl != NULL) {
		pFocusTextCtrl->Cut();
	}
}

void MainFrame::OnEditPaste(wxCommandEvent& event)
{
	wxTextCtrl *pFocusTextCtrl = GetFocusTextCtrl();
	if (pFocusTextCtrl != NULL) {
		pFocusTextCtrl->Paste();
	} 
}


void MainFrame::OnToolsExecuteQuery(wxCommandEvent& event)
{
	std::istringstream istr(std::string((const char*)(m_pEditWindow->GetValue().mb_str(wxConvUTF8))));
	if (m_pEE == 0 || !m_pEE->connectionOk()) {
		wxEmdrosErrorMessage(wxT("Error: The connection to the backend is not OK.\n")
				     wxT("You must use \"Tools\"-->\"New connection...\"\n")
				     wxT("to establish a proper connection."));
		return;
	} else {
		m_bExecuting = true;
		int nReturnCode = exec_stream(&istr, m_pEW);
		if (m_pOut->ShouldStop()) {
			wxEmdrosInfoMessage(wxT("You successfully stopped the process."));
			m_pOut->UnStop();
		}
		if (nReturnCode != 0) {
			wxEmdrosErrorMessage(wxT("Error: Something went wrong while executing query."));
		}
		m_bExecuting = false;
	}
	return;
}

void MainFrame::OnToolsStopQuery(wxCommandEvent& event)
{
	m_pOut->Stop();
}

void MainFrame::OnToolsNewConnection(wxCommandEvent& event)
{
	ConnectionData conndata;
	if (!wxGetApp().GetConnection(conndata)) {
		// This means that the user pressed 'cancel'.
	} else {
		m_connectionData = conndata;
		Connect();
	}
}

void MainFrame::OnToolsConfigure(wxCommandEvent& event)
{
	//wxEmdrosInfoMessage(wxT("Sorry. Not implemented yet.\nPlease close this dialog box, then\npress F1 to see the User's Guide about how to write\na *.cfg configuration file."));
	try {
		ConnectionWizard *pWizard = new ConnectionWizard(this);
		if (pWizard->RunIt()) {
			wxEmdrosInfoMessage(wxT("Finished successfully!"));
		} else {
			wxEmdrosInfoMessage(wxT("Aborted."));
		}
	} catch (EmdrosException e) {
		wxEmdrosInfoMessage(wxString(e.what().c_str(), wxConvUTF8));
	}
}


void MainFrame::OnFileExit(wxCommandEvent& event)
{
	Close();
}

wxTextCtrl *MainFrame::GetFocusTextCtrl(void)
{
	wxWindow *pFocusWindow = wxWindow::FindFocus();
	if (pFocusWindow == m_pResultsWindow) {
		//return m_pResultsWindow;
		return NULL;
	} else if (pFocusWindow == m_pEditWindow) {
		return m_pEditWindow;
	} else {
		return NULL;
	}
} 

void MainFrame::ClearTextCtrls(void)
{
	m_pEditWindow->Clear();
	m_pResultsWindow->Clear();
	this->Refresh();
}

void MainFrame::ClearOutputWindow(void)
{
	m_pResultsWindow->Clear();
	this->Refresh();
}

void MainFrame::ReadQuery()
{
	std::ifstream fin;
	fin.open(std::string((const char*)(m_strCurFileName.mb_str(wxConvUTF8))).c_str(), std::ios::in);
	if (!fin) {
		wxEmdrosErrorMessage(wxT("Could not open file."));
	} else {
		ClearTextCtrls();
		while (!fin.eof()) {
#define BUFFERSIZE (2048)
			char szBuffer[BUFFERSIZE];      
			fin.getline(szBuffer, BUFFERSIZE);
			m_pEditWindow->AppendText(wxString(szBuffer, wxConvUTF8));
			m_pEditWindow->AppendText(wxString(wxT("\n")));
		} 
		fin.close();
	}
}

void MainFrame::SaveTextCtrl(const wxString& filename, wxTextCtrl *pTextCtrl)
{
	std::ofstream fout;
	fout.open(std::string((const char*)(filename.mb_str(wxConvUTF8))).c_str(), std::ios::out);
	if (!fout) {
		wxEmdrosErrorMessage(wxT("Could not open file for writing."));
	} else  {
		wxString strQuery = pTextCtrl->GetValue();
		fout << std::string((const char*) strQuery.mb_str(wxConvUTF8));
	}

}

void MainFrame::SaveOutputAreaAsHTML(const wxString& filename)
{
	std::ofstream fout;
	std::string str_filename = std::string((const char*)(filename.mb_str(wxConvUTF8)));
	fout.open(str_filename.c_str(), std::ios::out);
	if (!fout) {
		wxEmdrosErrorMessage(wxT("Could not open file for writing."));
	} else  {
		wxBusyCursor busyCursor;
		m_pResultsWindow->toHTML(&fout, str_filename);
	}
}

void MainFrame::SaveQuery()
{
	SaveTextCtrl(m_strCurFileName, m_pEditWindow);
}

bool MainFrame::Connect()
{
	ClearSchemaTree(wxString(wxT("")));
	delete m_pConf;
	std::ostringstream ssout;
	m_pConf = parse_config_file(std::string((const char*)m_connectionData.m_strConfiguration.mb_str(wxConvUTF8)),
				    std::string(app_prefix()),
				    &ssout);
	if (m_pConf == 0) {
		(*m_pOut) << ssout.str();
		wxEmdrosErrorMessage(wxString(wxT("Error: Could not load configuration file:\n")) + m_connectionData.m_strConfiguration);
		ClearSchemaTree(wxString(wxT("Configuration not loaded")));
		this->Refresh();
		return false;
	} else {
		wxString strDB;
		if (m_connectionData.m_strDatabase.IsEmpty()) {
			if (m_pConf->has_key("database")) {
				strDB = wxString(m_pConf->getValues("database")[0].c_str(), wxConvUTF8);
			} else {
				wxEmdrosErrorMessage(wxString(wxT("Error: You must specify a database name,\n")
							      wxT("either in the connection dialog or in the\n")
							      wxT("configuration file (with the 'database' key).")));
				ClearSchemaTree(wxString(wxT("No database specified")));
				this->Refresh();
				return false;
			}
		} else {
			strDB = m_connectionData.m_strDatabase;
		}

		// Connect    
		delete m_pEE;
		
		// FIXME: This uses std::cout...
		m_pEE = new EmdrosEnv(kOKConsole,
				      kCSISO_8859_1, // FIXME: This should choose UTF8 sometimes...
				      std::string((const char*) m_connectionData.m_strHost.mb_str(wxConvUTF8)),
				      std::string((const char*) m_connectionData.m_strUser.mb_str(wxConvUTF8)),
				      std::string((const char*) m_connectionData.m_strPassword.mb_str(wxConvUTF8)),
				      std::string((const char*) strDB.mb_str(wxConvUTF8)),
				      m_connectionData.m_backend_kind);

		// Make new EmdrosWrapper
		delete m_pEW;
		m_pEW = new EmdrosWrapper(m_pEE, m_pConf, m_pOut);
		std::string message;
		if (!m_pEW->configurePOut(message)) {
			wxString wxStrMsg = wxString(message.c_str(), wxConvUTF8);
			wxEmdrosErrorMessage(wxString(wxT("Error: While configuring, I encountered the following problem:\n") + wxStrMsg));
		}

		// Initialize m_pResultsWindow
		m_pResultsWindow->InitializeViewMetrics(m_pConf, m_pOut->getNoOfInterlinearLines());
		m_pResultsWindow->Clear();

		// Set size and font name of input area
		if (m_pOut != 0) {
			wxTextAttr inputAreaTextAttr(*wxBLACK, wxNullColour,
						     wxFont(m_pOut->getInputAreaFontSize(), 
							    wxMODERN, wxNORMAL, wxNORMAL, 
							    FALSE, 
							    m_pOut->getInputAreaFontName()));
			inputAreaTextAttr.SetFlags(wxTEXT_ATTR_TEXT_COLOUR
						   | wxTEXT_ATTR_FONT_FACE
						   | wxTEXT_ATTR_FONT_SIZE
						   | wxTEXT_ATTR_FONT_WEIGHT
						   | wxTEXT_ATTR_FONT_UNDERLINE
						   | wxTEXT_ATTR_ALIGNMENT);
			m_pEditWindow->SetStyle(0, m_pEditWindow->GetLastPosition(), inputAreaTextAttr);
			m_pEditWindow->SetDefaultStyle(inputAreaTextAttr);
			// This is necessary on Windows, where the MQL parser otherwise complains
			// about a strange token 'character something'.
			if (m_pEditWindow->GetValue().empty()) {
				m_pEditWindow->Clear();
			}
		}
	
		// Check that connection is OK
		if (!m_pEE->connectionOk()) {
			wxString error_message(wxT("Error: Could not connect to backend.\n")
					       wxT("Maybe the username, password, hostname,\n")
					       wxT("or database name were wrong..."));
#ifdef __WXMSW__
			if (m_connectionData.m_backend_kind == kMySQL) {
				error_message += wxString(wxT("\n\nOr it may be that you do not have\n")
							  wxT("libmysql.dll in your path.\n")
							  wxT("You can copy it from your MySQL\n")
							  wxT("installation to the directory where this\n")
							  wxT("program is located, or to C:\\WINDOWS\\SYSTEM32\\."));
			}
#endif
					       wxEmdrosErrorMessage(error_message);
			ClearSchemaTree(wxString(wxT("Could not establish connection...")));
			this->Refresh();
			return false;
		}
		PopulateSchemaTree();
		m_pOut->print(std::string((const char*) (wxString(wxT("Successfully connected.\n")).mb_str(wxConvUTF8))));
		this->Refresh();
		return true;
	}
}

void MainFrame::OnUpdateUIExec(wxUpdateUIEvent& event)
{
	event.Enable(m_bExecuting);
}

void MainFrame::OnUpdateUINoExec(wxUpdateUIEvent& event)
{
	event.Enable(!m_bExecuting);
}

void MainFrame::ClearSchemaTree(wxString msg)
{
	m_pSchemaTree->DeleteAllItems();
	m_pSchemaTree->AddRoot(msg);
}

void MainFrame::PopulateSchemaTree(void)
{
	ClearSchemaTree(m_connectionData.m_strDatabase);
	std::ostringstream stringstream;
	if (!m_pEW->retrieveSchema(&stringstream)) {
		wxEmdrosErrorMessage(wxT("Error: Could not retrieve schema."));
	} else {
		EmdrosSchema *pSchema = m_pEW->getPSchema();
		if (pSchema != 0) {
			AddEnumerationsToSchemaTree(pSchema);
			AddObjectTypesToSchemaTree(pSchema);
			AddMonadSetsToSchemaTree(pSchema);
		}
	}
}


void MainFrame::AddEnumerationsToSchemaTree(EmdrosSchema *pSchema)
{
	wxTreeItemId EnumRoot = m_pSchemaTree->AppendItem(m_pSchemaTree->GetRootItem(),
							  wxT("Enumerations"));

	const EmdrosEnumerations& enums = pSchema->getEnums();
	if (enums.isEmpty()) {
		m_pSchemaTree->AppendItem(EnumRoot,
					  wxT("no enumerations defined..."));        
	} else {
		StringVec enum_names = enums.getEnumNames();
		for (StringVec::const_iterator enumci = enum_names.begin();
		     enumci != enum_names.end();
		     ++enumci) {
			std::string enumeration_name = *enumci;
    
			wxString EnumString = wxString(enumeration_name.c_str(), wxConvUTF8);
			wxTreeItemId this_enumeration_root = 
				m_pSchemaTree->AppendItem(EnumRoot, EnumString);    
    
			StringVec enum_consts = enums.getEnumConstantNames(enumeration_name);
			for (StringVec::const_iterator enum_const_ci = enum_consts.begin();
			     enum_const_ci != enum_consts.end();
			     ++enum_const_ci) {
				std::string enum_const_name = *enum_const_ci;
				wxString EnumConstString = wxString(enum_const_name.c_str(), wxConvUTF8);
				m_pSchemaTree->AppendItem(this_enumeration_root, 
							  EnumConstString);    
			}
		}
	}
}

bool MainFrame::ExecuteEmdrosString(std::string query)
{
	return m_pEW->executeString(query);
}



void MainFrame::AddObjectTypesToSchemaTree(EmdrosSchema *pSchema)
{
	wxTreeItemId OTRoot = m_pSchemaTree->AppendItem(m_pSchemaTree->GetRootItem(),
							wxT("Object Types"));
	const EmdrosObjectTypes& object_types = pSchema->getObjectTypes();
	if (object_types.isEmpty()) {
		m_pSchemaTree->AppendItem(OTRoot,
					  wxT("no object types defined..."));        
	} else {
		StringVec OT_names = object_types.getObjectTypeNames();
		for (StringVec::const_iterator otci = OT_names.begin();
		     otci != OT_names.end();
		     ++otci) {
			std::string object_type_name = *otci;
    
			wxString OTString = wxString(object_type_name.c_str(), wxConvUTF8);
			wxTreeItemId this_OT_root = 
				m_pSchemaTree->AppendItem(OTRoot, OTString);    
      
			std::list<FeatureNameAndType> features 
				= object_types.getFeaturesOfObjectType(object_type_name);

			for (std::list<FeatureNameAndType>::const_iterator featntci = features.begin();
			     featntci != features.end();
			     ++featntci) {
				std::string feature_name = featntci->name;
				std::string feature_type_name = featntci->type;
				std::string feature_string = feature_name + " : " + feature_type_name;
				wxString FeatureString = wxString(feature_string.c_str(), wxConvUTF8);
				wxTreeItemId feature_root = m_pSchemaTree->AppendItem(this_OT_root, 
										      FeatureString);
				if (EmdrosObjectTypes::FeatureTypeIsEnum(*featntci)) {
					const EmdrosEnumerations& enums = pSchema->getEnums();
					StringVec enum_consts = enums.getEnumConstantNames(feature_type_name);
					for (StringVec::const_iterator enum_const_ci = enum_consts.begin();
					     enum_const_ci != enum_consts.end();
					     ++enum_const_ci) {
						std::string enum_const_name = *enum_const_ci;
						wxString EnumConstString = wxString(enum_const_name.c_str(), wxConvUTF8);
						m_pSchemaTree->AppendItem(feature_root, EnumConstString);    
					}
				}
			}
		}
	}
}


void MainFrame::AddMonadSetsToSchemaTree(EmdrosSchema *pSchema)
{
	wxTreeItemId MQLSetRoot = m_pSchemaTree->AppendItem(m_pSchemaTree->GetRootItem(),
							    wxT("Monad sets"));
  
	const EmdrosMonadSets& monad_sets = pSchema->getMonadSets();
	if (monad_sets.isEmpty()) {
		m_pSchemaTree->AppendItem(MQLSetRoot, wxString(wxT("No monad sets defined...")));
	} else {
		for (EmdrosMonadSets::const_iterator msci = monad_sets.begin();
		     msci != monad_sets.end();
		     ++msci) {
			std::string monad_set_name = msci->first;
			SetOfMonads som = msci->second;
			wxString MonadSetNameString = wxString(monad_set_name.c_str(), wxConvUTF8);
			wxTreeItemId this_monad_set_root = 
				m_pSchemaTree->AppendItem(MQLSetRoot, MonadSetNameString);    
      
			wxString MonadSetSetString = wxString(som.toString().c_str(), wxConvUTF8);
			m_pSchemaTree->AppendItem(this_monad_set_root, 
						  MonadSetSetString);    
		}
	}
}

void MainFrame::readConfig()
{
	// Create it if it does not exist;
	// use the name set by the app's constructor's call to
	// SetAppName for the root of the tree.
	wxConfigBase *config = wxConfigBase::Get();
	
	// Flush all current changes to disk/registry
	config->Flush();
	
	config->SetPath(wxT("/QueryTool/"));
	
	wxString myString;


	if (config->Read(wxT("QueryFileDir"), &myString)) {
		m_strQueryFileDir = myString;
	} else {
		m_strQueryFileDir = ::wxGetHomeDir();
	}
}

void MainFrame::writeConfig()
{
	// Create it if it does not exist;
	// use the name set by LTCApp::LTCApp's call to
	// SetAppName for the root of the tree.
	wxConfigBase *config = wxConfigBase::Get();

	config->SetPath(wxT("/QueryTool/"));

	config->Write(wxT("QueryFileDir"), m_strQueryFileDir);

	// Flush all current changes to disk/registry
	config->Flush();
}




void MyTextCtrl::OnChar(wxKeyEvent& event)
{
	if (event.GetKeyCode() == WXK_TAB) {
		WriteText(wxT("\t"));
	} else {
		event.Skip();
	}
}

